
#define DS1302_SECONDS           0x80
#define DS1302_MINUTES           0x82
#define DS1302_HOURS             0x84
#define DS1302_DATE              0x86
#define DS1302_MONTH             0x88
#define DS1302_DAY               0x8A
#define DS1302_YEAR              0x8C
#define DS1302_ENABLE            0x8E
#define DS1302_TRICKLE           0x90
#define DS1302_CLOCK_BURST       0xBE
#define DS1302_CLOCK_BURST_WRITE 0xBE
#define DS1302_CLOCK_BURST_READ  0xBF
#define DS1302_RAMSTART          0xC0
#define DS1302_RAMEND            0xFC
#define DS1302_RAM_BURST         0xFE
#define DS1302_RAM_BURST_WRITE   0xFE
#define DS1302_RAM_BURST_READ    0xFF

#define DS1302_D0 0
#define DS1302_D1 1
#define DS1302_D2 2
#define DS1302_D3 3
#define DS1302_D4 4
#define DS1302_D5 5
#define DS1302_D6 6
#define DS1302_D7 7

#define DS1302_READBIT DS1302_D0 // READBIT=1: read instruction
#define DS1302_RC DS1302_D6
#define DS1302_CH DS1302_D7   // 1 = Clock Halt, 0 = start
#define DS1302_AM_PM DS1302_D5 // 0 = AM, 1 = PM
#define DS1302_12_24 DS1302 D7 // 0 = 24 hour, 1 = 12 hour

#define DS1302_WP DS1302_D7   // 1 = Write Protect, 0 = enabled

#define DS1302_ROUT0 DS1302_D0
#define DS1302_ROUT1 DS1302_D1
#define DS1302_DS0   DS1302_D2
#define DS1302_DS1   DS1302_D2
#define DS1302_TCS0  DS1302_D4
#define DS1302_TCS1  DS1302_D5
#define DS1302_TCS2  DS1302_D6
#define DS1302_TCS3  DS1302_D7


typedef struct ds1302_struct
{
  uint8_t Seconds:4;      // low decimal digit 0-9
  uint8_t Seconds10:3;    // high decimal digit 0-5
  uint8_t CH:1;       // CH = Clock Halt
  
  uint8_t Minutes:4;
  uint8_t Minutes10:3;
  uint8_t reserved1:1;
  
  union
  {
    struct
    {
      uint8_t Hour:4;
      uint8_t Hour10:2;
      uint8_t reserved2:1;
      uint8_t hour_12_24:1; // 0 for 24 hour format
    } h24;
    struct
    {
      uint8_t Hour:4;
      uint8_t Hour10:1;
      uint8_t AM_PM:1;      // 0 for AM, 1 for PM
      uint8_t reserved2:1;
      uint8_t hour_12_24:1; // 1 for 12 hour format
    } h12;
  };
  uint8_t Date:4;
  uint8_t Date10:2;
  uint8_t reserved3:2;
  
  uint8_t Month:4;
  uint8_t Month10:1;
  uint8_t reserved4:3;
  
  uint8_t Day:3;
  uint8_t reserved5:5;
  
  uint8_t Year:4;
  uint8_t Year10:4;
  
  uint8_t reserved6:7;
  uint8_t WP:1;             // WP = Write Protect
};

 ds1302_struct relojDS;


void DS1302_clock_burst_read( uint8_t *p);
void DS1302_clock_burst_write( uint8_t *p);
void _DS1302_start(void);
void _DS1302_stop(void);
void _DS1302_togglewrite(uint8_t data, uint8_t release);
uint8_t _DS1302_toggleread(void);

 

void resetChip()
{
  memset ((char *) &relojDS, 0, sizeof(relojDS));
  relojDS.Seconds = 0;
  relojDS.Seconds10 = 0;
  relojDS.CH = 0;        // 1 for Clock Halt, 0 to run;
  relojDS.Minutes = 0;
  relojDS.Minutes10 = 0;
  relojDS.h24.Hour = 0;
  relojDS.h24.Hour10 = 0;
  relojDS.h24.hour_12_24 = 0; // 0 for 24 hour format
  relojDS.Date = 1;
  relojDS.Date10 = 0;
  relojDS.Month = 1;
  relojDS.Month10 = 0;
  relojDS.Day = 1;
  relojDS.Year = 1;
  relojDS.Year10 = 1;
  relojDS.WP = 0;  
  DS1302_clock_burst_write( (uint8_t *) &relojDS);
}




void DS1302_clock_burst_read( uint8_t *p)
{
  int i;

  _DS1302_start();
  _DS1302_togglewrite( DS1302_CLOCK_BURST_READ, true);  
  for (i=0; i<8; i++)    *p++ = _DS1302_toggleread();  
  _DS1302_stop();
}


void DS1302_clock_burst_write( uint8_t *p)
{
  int i;

  _DS1302_start();
  _DS1302_togglewrite( DS1302_CLOCK_BURST_WRITE, false);  
  for (i=0; i<8; i++)    _DS1302_togglewrite( *p++, false);  
  _DS1302_stop();
}

uint8_t DS1302_read(int address)
{
  uint8_t data;

  bitSet (address, DS1302_READBIT);  
  _DS1302_start();
  _DS1302_togglewrite (address, true);  
  data = _DS1302_toggleread ();
  _DS1302_stop();

  return (data);
}

void DS1302_write(int address, uint8_t data)
{
  bitClear (address, DS1302_READBIT);  

  _DS1302_start();
  _DS1302_togglewrite (address, false);
  _DS1302_togglewrite (data, false);
  _DS1302_stop();  
}

void _DS1302_start(void)
{
  digitalWrite (DS1302_CE, LOW); // default, not enabled
  pinMode (DS1302_CE, OUTPUT);  
  digitalWrite (DS1302_SCLK, LOW); // default, clock low
  pinMode (DS1302_SCLK, OUTPUT);
  pinMode (DS1302_IO, OUTPUT);
  digitalWrite (DS1302_CE, HIGH); // start the session
  delayMicroseconds(4);           // tCC = 4us
}

void _DS1302_stop(void)
{
  digitalWrite (DS1302_CE, LOW);
  delayMicroseconds(4);           // tCWH = 4us
}


uint8_t _DS1302_toggleread(void)
{
  uint8_t i, data;

  data = 0;
  for (i = 0; i <= 7; i++)
  {
    digitalWrite (DS1302_SCLK, HIGH);
    delayMicroseconds(1);
    digitalWrite (DS1302_SCLK, LOW);
    delayMicroseconds(1);        // tCL=1000ns, tCDD=800ns
    bitWrite (data, i, digitalRead(DS1302_IO));
  }
  return (data);
}


void _DS1302_togglewrite(uint8_t data, uint8_t release)
{
  int i;

  for (i = 0; i <= 7; i++)
  {
    digitalWrite (DS1302_IO, bitRead(data, i));  
    delayMicroseconds(1);     // tDC = 200ns
    digitalWrite (DS1302_SCLK, HIGH);    
    delayMicroseconds(1);     // tCH = 1000ns, tCDH = 800ns
    if (release && i == 7)
    {
      pinMode (DS1302_IO, INPUT);
      digitalWrite (DS1302_IO, LOW); // remove any pull-up  
    }
    else
    {
      digitalWrite (DS1302_SCLK, LOW);
      delayMicroseconds(1);       // tCL=1000ns, tCDD=800ns
    }
  }
}


//___________________________________________
